package com.vaadin.starter.skeleton.smartgrid2;

import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.checkbox.Checkbox;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.notification.Notification;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.data.provider.ListDataProvider;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.vaadin.flow.data.renderer.Renderer;
import com.vaadin.flow.function.SerializableFunction;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.server.Command;
import com.vaadin.flow.server.VaadinService;
import com.vaadin.starter.skeleton.smartgrid2.polymerccoloreddiv.PolymerColoredDiv;

import java.util.ArrayList;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;


public class SmartGrid2 extends VerticalLayout {


    private Grid<ColoredRow> component;
    private ArrayList<ColoredRow> dataList;
    private ListDataProvider<ColoredRow> dataProvider;


    public SmartGrid2(int initialRowCount, int updatePeriodSeconds) {

        this.setHeightFull();
        this.setWidthFull();

        component = new Grid<>();
        component.setSizeFull();
        component.setSelectionMode(Grid.SelectionMode.SINGLE);
        component.setColumnReorderingAllowed(true);

        // Columns
        provideGridColumns();

        // DataProvider
        dataList = new ArrayList<>();
        dataProvider = new ListDataProvider<>(dataList);
        component.setDataProvider(dataProvider);

        // Initial data
        for (int row = 0; row < initialRowCount; row++) {
            dataList.add(new ColoredRow());
        }

        // Periodic reloader
        Executors.newSingleThreadScheduledExecutor().scheduleAtFixedRate(() -> reloadGrid(), 0, updatePeriodSeconds, TimeUnit.SECONDS);

        add(component);

    }


    /**
     * Настройка колонок грида
     */
    private void provideGridColumns() {
        component.addColumn(createColorRenderer001()).setSortable(true).setResizable(true);
        component.addColumn(createColorRenderer002()).setSortable(true).setResizable(true);
        component.addColumn(createColorRenderer003()).setSortable(true).setResizable(true);
        component.addColumn(createColorRenderer004()).setSortable(true).setResizable(true);
        component.addColumn(createColorRenderer005()).setSortable(true).setResizable(true);
    }


    private Renderer<ColoredRow> createColorRenderer001() {
        ComponentRenderer<PolymerColoredDiv, ColoredRow> componentRenderer = new ComponentRenderer<>(new SerializableFunction<ColoredRow, PolymerColoredDiv>() {
            @Override
            public PolymerColoredDiv apply(ColoredRow coloredRow) {
                String colorString = SmartGrid2.this.getColorString(coloredRow.getC001());
                int threadId = (int) Thread.currentThread().getId();

                if (VaadinService.getCurrent() == null)
                    System.out.println("Detected VaadinService.getCurrent==null, ThreadId: " + threadId);
                else
                    System.out.println("ThreadId: " + threadId);
                return new PolymerColoredDiv(colorString, threadId);
            }
        });
        return componentRenderer;
    }


    private Renderer<ColoredRow> createColorRenderer002() {
        ComponentRenderer<PolymerColoredDiv, ColoredRow> componentRenderer = new ComponentRenderer<>(new SerializableFunction<ColoredRow, PolymerColoredDiv>() {
            @Override
            public PolymerColoredDiv apply(ColoredRow coloredRow) {
                String colorString = SmartGrid2.this.getColorString(coloredRow.getC002());
                int threadId = (int) Thread.currentThread().getId();

                if (VaadinService.getCurrent() == null)
                    System.out.println("Detected VaadinService.getCurrent==null, ThreadId: " + threadId);
                else
                    System.out.println("ThreadId: " + threadId);

                return new PolymerColoredDiv(colorString, threadId);
            }
        });
        return componentRenderer;
    }


    private Renderer<ColoredRow> createColorRenderer003() {
        ComponentRenderer<PolymerColoredDiv, ColoredRow> componentRenderer = new ComponentRenderer<>(new SerializableFunction<ColoredRow, PolymerColoredDiv>() {
            @Override
            public PolymerColoredDiv apply(ColoredRow coloredRow) {
                String colorString = SmartGrid2.this.getColorString(coloredRow.getC003());
                int threadId = (int) Thread.currentThread().getId();

                if (VaadinService.getCurrent() == null)
                    System.out.println("Detected VaadinService.getCurrent==null, ThreadId: " + threadId);
                else
                    System.out.println("ThreadId: " + threadId);

                return new PolymerColoredDiv(colorString, threadId);
            }
        });
        return componentRenderer;
    }


    private Renderer<ColoredRow> createColorRenderer004() {
        ComponentRenderer<PolymerColoredDiv, ColoredRow> componentRenderer = new ComponentRenderer<>(new SerializableFunction<ColoredRow, PolymerColoredDiv>() {
            @Override
            public PolymerColoredDiv apply(ColoredRow coloredRow) {
                String colorString = SmartGrid2.this.getColorString(coloredRow.getC004());
                int threadId = (int) Thread.currentThread().getId();

                if (VaadinService.getCurrent() == null)
                    System.out.println("Detected VaadinService.getCurrent==null, ThreadId: " + threadId);
                else
                    System.out.println("ThreadId: " + threadId);

                return new PolymerColoredDiv(colorString, threadId);
            }
        });
        return componentRenderer;
    }


    private Renderer<ColoredRow> createColorRenderer005() {
        ComponentRenderer<PolymerColoredDiv, ColoredRow> componentRenderer = new ComponentRenderer<>(new SerializableFunction<ColoredRow, PolymerColoredDiv>() {
            @Override
            public PolymerColoredDiv apply(ColoredRow coloredRow) {
                String colorString = SmartGrid2.this.getColorString(coloredRow.getC005());
                int threadId = (int) Thread.currentThread().getId();
                if (VaadinService.getCurrent() == null)
                    Notification.show("Detected VaadinService.getCurrent==null, ThreadId: " + threadId, 60000, Notification.Position.BOTTOM_CENTER);

                return new PolymerColoredDiv(colorString, threadId);
            }
        });
        return componentRenderer;
    }


    private void reloadGrid() {
        dataList.forEach(ColoredRow::updateAllColumns);
        //dataList.add(new ColoredRow()); // Add one more row each time

        Optional<UI> optionalUI = component.getUI();
        optionalUI.ifPresent(new Consumer<UI>() {
            @Override
            public void accept(UI ui) {
                ui.access(new Command() {
                              @Override
                              public void execute() {
                                  component.getDataProvider().refreshAll();
                              }
                          }
                );
            }
        });
    }


    private String getColorString(int color) {
        String result = "#000000";
        int value = 0xff000000 | color;

        int red = (value >> 16) & 0xFF;
        String redString = Integer.toHexString(red);
        redString = redString.length() < 2 ? "0" + redString : redString;

        int green = (value >> 8) & 0xFF;
        String greenString = Integer.toHexString(green);
        greenString = greenString.length() < 2 ? "0" + greenString : greenString;

        int blue = (value >> 0) & 0xFF;
        String blueString = Integer.toHexString(blue);
        blueString = blueString.length() < 2 ? "0" + blueString : blueString;

        //int alpha = (value >> 24) & 0xff;

        result = "#" + redString + greenString + blueString;

        return result;
    }


}